import { Injectable } from '@angular/core';
import { Meta, Title } from '@angular/platform-browser';
import { environment } from '@env/environment';
import { ActivatedRoute, NavigationEnd, Router } from '@angular/router';
import { filter, map, mergeMap } from 'rxjs/operators';

@Injectable()
export class SEOService {

  private defaultTitle = 'DiscoTrip';
  private defaultDescription = 'Nothing';
  private defaultNoIndex = false;
  private defaultNoFollow = false;

  constructor( private title: Title, private meta: Meta, private router: Router, private activatedRoute: ActivatedRoute ) {}

  public start() {
    this.meta.updateTag( { name: 'og:locale', contest: 'fr_FR' } );
    this.meta.updateTag( { name: 'og:site_name', contest: environment.appName } );
    this.meta.updateTag( { name: 'og:url', content: location.href } );
    this.routerListener();
  }

  private updateTitle( title: string ) {
    this.title.setTitle( title );
    this.meta.updateTag( { name: 'og:title', content: title } );
  }

  private updateDescription( desc: string ) {
    this.meta.updateTag( { name: 'description', content: desc } );
    this.meta.updateTag( { name: 'og:description', content: desc } );
  }

  private updateIndexFollow( noindex: boolean, nofollow: boolean ) {
    let content = '';

    content += ( !noindex ? '' : 'noindex' );
    content += ( content.length > 0 ? ', ' : '' );
    content += ( !nofollow ? '' : 'nofollow' );

    if ( content.length > 0 ) {
      this.meta.updateTag( { name: 'robots', content } );
      this.meta.updateTag( { name: 'googlebot', content } );
    }
  }

  private setMetas( event: any ) {
    event.title = ( !event.title ? this.defaultTitle : event.title );
    event.description = ( !event.description ? this.defaultDescription : event.description );
    event.noindex = ( !event.noindex ? this.defaultNoIndex : event.noindex );
    event.nofollow = ( !event.nofollow ? this.defaultNoFollow : event.nofollow );
    this.updateTitle( event.title + ' | C2A' );
    this.updateDescription( event.description );
    this.updateIndexFollow( event.noindex, event.nofollow );
  }

  private routerListener() {
    this.router.events
      .pipe(
        filter( ( event ) => event instanceof NavigationEnd ),
        map( () => this.activatedRoute ),
        map( ( route ) => {
          while ( route.firstChild ) {
            route = route.firstChild;
          }
          return route;
        } ),
        filter( ( route ) => route.outlet === 'primary' ),
        mergeMap( ( route ) => route.data )
      ).subscribe( ( event ) => {
      this.setMetas( event );
    } );
  }

}
